namespace EditorRouteUI
{
    namespace TabIO
    {

    
    
        bool bIsSavedRoutesFileListDirty = true;
        string SavedRoutesFolder;
        string[]@ SavedRoutesFileList;

        uint32 SelectedMapIndex;
        dictionary MapFoldersDict;
        array<string>@ MapFoldersList;

        void Init()
		{
            if (bIsSavedRoutesFileListDirty)
            {
                SavedRoutesFolder = IO::FromStorageFolder("Saved/Routes/");
                IO::CreateFolder(SavedRoutesFolder, true);

                @SavedRoutesFileList = IO::IndexFolder(SavedRoutesFolder, true);

                // Assuming SavedRoutesFolder is already populated and may contain either slash type
                string root = SavedRoutesFolder; // e.g., "C:/Users/Username/OpenplanetNext/PluginStorage/YourPluginIdentifier/Saved/Routes/"

                // Escape slashes for regex use and replace with a pattern that matches both slash types
                string escapedRoot = Regex::Replace(root, "[\\\\/]", "[\\\\\\\\/]", Regex::Flags::ECMAScript);

                // Construct the regex pattern dynamically to be agnostic to slash types
                string pattern = "^" + escapedRoot + "([^\\\\/]+)[\\\\/]([^\\\\/]+)\\.json$";

                MapFoldersDict.DeleteAll();
                for (uint32 i = 0; i < SavedRoutesFileList.Length; i++)
                {
                    trace(SavedRoutesFileList[i]);
                    string[]@ matches = Regex::Search(SavedRoutesFileList[i], pattern);

                    if (matches.Length > 2) 
                    {
                        string mapName = matches[1];
                        string fileName = matches[2];
                        
                        trace("mapName: "+mapName);
                        trace("fileName: "+fileName);

                        if (!MapFoldersDict.Exists(mapName))
                        {
                            array<string> files = { fileName };
                            MapFoldersDict[mapName] = files;
                        }
                        else 
                        {
                            array<string> @files;
                            @files = cast<array<string>>(MapFoldersDict[mapName]);
                            files.InsertLast(fileName);
                        }
                    }
                }
                SelectedMapIndex = 0;
                @MapFoldersList = MapFoldersDict.GetKeys();

                bIsSavedRoutesFileListDirty = false;
            }
		}

		void Draw()
		{
			Init();

            if(UI::BeginTabItem("IO", UI::TabItemFlags::NoReorder))
			{
                const vec2 contentRegion = UI::GetContentRegionAvail();

                if (UI::Button(Icons::Refresh))
                {
                    bIsSavedRoutesFileListDirty = true;
                }

                if (UI::BeginCombo("Choose Map", SelectedMapIndex < MapFoldersList.Length ? MapFoldersList[SelectedMapIndex] : "None"))
                {
                    for (uint32 i = 0; i < MapFoldersList.Length; i++)
                    {
                        const bool bIsSelected = (SelectedMapIndex == i);
                        if (UI::Selectable(MapFoldersList[i], bIsSelected)) { SelectedMapIndex = i; }
                        if (bIsSelected) { UI::SetItemDefaultFocus(); }
                    }
                    UI::EndCombo();
                }
                UI::SameLine();

				const int32 tableFlags = UI::TableFlags::RowBg | UI::TableFlags::SizingStretchProp | UI::TableFlags::BordersInnerV | UI::TableFlags::ScrollY;
                const int32 treeNodeFlags = UI::TreeNodeFlags::SpanFullWidth | /* ImGuiTreeNodeFlags_SpanAllColumns */ (1 << 13);

				if (UI::BeginTable("UITabTable", 3, tableFlags, contentRegion))
				{
                    int32 _sv = 0;
					_sv += PushStyleVar(UI::StyleVar::FramePadding, vec2(8., 0.));

					UI::TableSetupColumn("File", UI::TableColumnFlags::WidthStretch, 0.25f);
					UI::TableSetupColumn("Value", UI::TableColumnFlags::WidthStretch, 0.5f);
					UI::TableSetupColumn("Action", UI::TableColumnFlags::WidthStretch, 0.25f);

                    UI::TableHeadersRow();

                    // Show Files as hierarchical tree view here

					PopStyleVar(_sv);
					UI::EndTable();
                }

                // ---------------------------------------------------------------
                // Organize files by subfolders (Map Names)
                // dictionary mapFolders;
                // for (uint i = 0; i < SavedRoutesFileList.length(); i++)
                // {
                //     string filePath = SavedRoutesFileList[i];
                //     string folderName = IO::GetParentPath(filePath).Replace(SavedRoutesFolder, "");
                //     string fileName = IO::GetFileName(filePath);
                    
                //     // Filter out non-JSON files
                //     if (!fileName.EndsWith(".json")) continue;

                //     if (!mapFolders.exists(folderName))
                //     {
                //         array<string> files = {fileName};
                //         mapFolders[folderName] = files;
                //     }
                //     else
                //     {
                //         array<string>@ files;
                //         mapFolders.get(folderName, @files);
                //         files.insertLast(fileName);
                //     }
                // }

                // Draw Table Tree
                // array<string> keys = mapFolders.getKeys();
                // keys.sortAsc(); // Sort folders alphabetically; adjust as needed
                // string currentMapName = GetCurrentMapName();
                // if (keys.find(currentMapName) >= 0)
                // {
                //     // Move the current map to the first position
                //     keys.removeAt(keys.find(currentMapName));
                //     keys.insertAt(0, currentMapName);
                // }

                // for (uint i = 0; i < keys.length(); i++)
                // {
                //     string mapName = keys[i];
                //     array<string>@ files;
                //     mapFolders.get(mapName, @files);

                //     if (UI::TreeNode(mapName))
                //     {
                //         for (uint j = 0; j < files.length(); j++)
                //         {
                //             UI::TableNextRow();
                //             UI::TableSetColumnIndex(1);
                //             UI::Text(files[j]); // File name

                //             UI::TableSetColumnIndex(2);
                //             if (UI::Button("Load##" + files[j]))
                //             {
                //                 // Load action
                //             }
                //             UI::SameLine();
                //             if (UI::Button("Delete##" + files[j]))
                //             {
                //                 // Delete action
                //             }
                //         }
                //         UI::TreePop();
                //     }
                // }

                // UI::EndTable();
                // ---------------------------------------------------------------

				UI::EndTabItem();
            }
        }
    }
}